/******************************************************************************
 * eiwic - Extensible Ircbot Written In C
 * Copyright (C) Hannes Gruler <lordi@styleliga.org>
 *
 * eiwic.h: Main header file.
 ****************************************************************************/

#ifndef _EIWIC_H_INCLUDED
#define _EIWIC_H_INCLUDED

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <setjmp.h>

/*
#include <sys/types.h>
#include <sys/ioctl.h>
#include <arpa/inet.h>
#include <dlfcn.h>
*/

typedef unsigned char *STRING;

#include "dlist.h"
#include "users.h"
#include "servers.h"

#ifndef NAME
#define NAME     "eiwic"
#endif
#ifndef VERSION
#define VERSION  "1.0.14"
#endif
#define AUTHOR   "Hannes Gruler <lordi@styleliga.org>"


//#define HTTP     struct eiwic_http_request
#define MSG      struct eiwic_ircmsg
#define PLUGIN   struct eiwic_plugin
#define TRIGGER  struct eiwic_plugin_trigger
#define TIMER    struct eiwic_plugin_timer
//#define CONN     struct eiwic_conn
#define GLOBAL   struct eiwic_glob
#define CHANNEL  struct eiwic_channel
#define SETTING  struct eiwic_setting
#define MODLINK  struct eiwic_modlink
#define USER     struct eiwic_user

#define SEGWARN(module,routine) \
	log(LOG_WARNING, "Segmentation fault in module %s.%s()",\
		(module)->name, routine);
#define MIN(x1,x2) \
	(((x1)<(x2))?(x1):(x2))

GLOBAL;
MSG;
TRIGGER;

typedef int   (*e_trigger_f)(char *, MSG *);
typedef void  (*e_timer_f)(void *);

CHANNEL {
	u_char name[128];
};

MSG {
	u_short cmd_num;
	u_char is_server;
	u_char cmd[10], to[50], args[512];
	u_char nick[30], username[30], host[512];
	u_char server[64];
};


PLUGIN {
	u_char filename[255];
	u_char name[20];
	char inited;
	void *handle;
	void *modlink;
	
	int (*ep_init)(GLOBAL *, PLUGIN *);
	int (*ep_main)();
	int (*ep_deinit)(void);
	int (*ep_parse)(MSG *);
	int (*ep_trigger)(TRIGGER *, MSG *);
};

TRIGGER {
	u_char trigger[30];
	e_trigger_f func;
	u_int flags;
	PLUGIN *plugin;
};

TIMER {
	u_long time;
	e_timer_f func;
	void *data;
	PLUGIN *plugin;
};

SETTING {
	char label[64];
	int type;
	union {
		char *d_string;
		float d_float;
		long d_int;
	} data;
};

GLOBAL {
	u_int	var_socket;
	u_char	quit, logged_in;
	u_char	botnick[30];
	
	u_char	vvd_section[40];
	u_char	*vvd_param;
	u_char	*b_send;

	u_char  mod_need_init;
	sigjmp_buf jmp_past;
	u_char jump_back;

	int		time_launched;

	int	var_bytes;

	int var_daemon;
	int var_log;
	int var_verbose;
	int var_noconnect;
	
	u_char var_logfilename[128];

	FILE *var_logfile;

	void *modlink;

	SERVER *server, *proxy;

	DList
		plugins,
		triggers,
		timers,
		connections,
		channels,
		servers,
		users,
		settings;

};

extern GLOBAL *e_global;
extern PLUGIN *plugin;

#endif /* _EIWIC_H_INCLUDED */
