#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <arpa/inet.h>
#include <ctype.h>
#include <dlfcn.h>
#include <time.h>
#include <errno.h>

#include "eiwic.h"
#include "plugins.h"

#define DATAFILE "spiegel.de.dat"
#define DELAY_MIN 10 /* how many minutes to wait between the updates */
#define NEWS_COUNT 3 /* how many news should be posted maximally */
#define POS_IDENT_STR "</STYLE>\n<div class=\"mark\">\n<p><b>"

EP_SETNAME("spiegel");

CHANNEL *post_chan;

void spiegel_retrieve(void *);

char *html_cleanup(char *string)
{
	char *p=string, rp, length;

	while (*p)
	{
		rp = '\0';
		length = 0;

		if (*p == '&')
		{
			if (strncmp(p+1, "uuml;", length=5) == 0) rp = '';
			else if (strncmp(p+1, "auml;", length=5) == 0) rp = '';
			else if (strncmp(p+1, "ouml;", length=5) == 0) rp = '';
			else if (strncmp(p+1, "quot;", length=5) == 0) rp = '\"';
			else if (strncmp(p+1, "Auml;", length=5) == 0) rp = '';
			else if (strncmp(p+1, "Ouml;", length=5) == 0) rp = '';
			else if (strncmp(p+1, "Uuml;", length=5) == 0) rp = '';
			else if (strncmp(p+1, "szlig;", length=6) == 0) rp = '';
			else length=0;
		}
		else if (*p == '<')
		{
			/* not very safe, but I must now learn for english test */
			length = (int)(strchr(p, '>') - p) + 1;
		}

		if (length > 0)
		{
			if (rp)
			{			
				memmove(p, p+length, strlen(p+length)+1);
				*p = rp;
			}
			else
				memmove(p, p+length, strlen(p+length)+1);
		}

		++p;
	}

	return string;
}

int spiegel_reply(char *retrieved, void *data)
{	
	char *a=retrieved, *p, *url, *headline, buf[1024], found=0;
	int i, cur = 0;
	FILE *dat;
	
	dat = fopen(DATAFILE, "a+");

	if (!dat)
	{
		mlink->log(LOG_WARNING, "Couldn't open "DATAFILE": %s", strerror(errno));
		return 0;
	}
	
#ifdef VV_DEBUG
	mlink->log(LOG_DEBUG, "%d bytes recieved, fd("DATAFILE"):%d", strlen(retrieved), dat);
#endif
	
	a = strstr(retrieved, POS_IDENT_STR);

	if (a == NULL)
		return 0;

	a += strlen(POS_IDENT_STR);
	
	while ((p = strstr(a, ">\n")) != NULL && cur < NEWS_COUNT)
	{
		fseek(dat, 0, SEEK_SET);
		
		p += 2;

		if (strncmp(p, "<a href=\"", 9) == 0)
		{
			p += 9;
			url = p;
	
			if (!(headline = strstr(url, "\">")))
				break;
		
			*headline = 0;
			headline += 2;

			if (!(a = strstr(headline, "</a>")))
				break;

			*a = 0;
			a += 1;
		
			while (fgets(buf, sizeof(buf) - 1, dat) && found == 0) {			
				for (i = 0; buf[i] == url[i] || buf[i] == ';'; ++i)
					if (buf[i] == ';' || url[i] == ';') {
						found = 1;
						break;
					}
			}

			if (found)
			{
				/* fprintf(stderr, "found: %s\n", url); */
			}
			else
			{			
				mlink->log(LOG_DEBUG, "PRIVMSG %s :%s: http://www.spiegel.de%s",
						((CHANNEL *)e_global->channels.head->data)->name,
						html_cleanup(headline), url);

				fprintf(dat, "%s;%s\n", url, headline);			
				mlink->plug_sendf("PRIVMSG %s :%s: http://www.spiegel.de%s\r\n",
						((CHANNEL *)e_global->channels.head->data)->name,
						html_cleanup(headline), url);
			}

			++cur;
		}
	}
	
	fflush(dat);
	fclose(dat);

	mlink->plug_timer_reg(time(NULL) + 60*DELAY_MIN, spiegel_retrieve, NULL);

	return 1;
}

void spiegel_retrieve(void *data)
{	
	mlink->conn_http_get("http://www.spiegel.de/schlagzeilen/", spiegel_reply, NULL);	
}


/* plugin main function */
int ep_main()
{
	mlink->plug_timer_reg(time(NULL)+15, spiegel_retrieve, NULL);
	
	return 1;
}
